package jsdp.app.routing.topologies;

public class Topology {
   int N;
   int[][] connectivity;
   double[][] distance;
   
   public Topology(int N, int[][] connectivity, double[][] distance){
      this.N = N;
      this.connectivity = connectivity;
      this.distance = distance;
   }
   
   public int getN(){
      return this.N;
   }
   
   public int[][] getConnectivity(){
      return this.connectivity;
   }
   
   public double[][] getDistance(){
      return this.distance;
   }
   
   public static Topology getTopology(int topologyNumber){
      switch(topologyNumber){
         case 0:
            return topology_0();
         case 1:
            return topology_1();
         case 2:
            return topology_2();
         case 3: 
            return topology_3();
         case 4:
            return topology_4();
         case 5:
            return topology_5();   
         default:
            throw new NullPointerException("Inexistent topology");
      }
   }
   
   private static Topology topology_0(){
      int N=10;
      int[][] connectivity = new int[][]{
            {1, 1, 0, 0, 0, 0, 0, 0, 0, 0},
            {1, 0, 1, 1, 0, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 1, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 1, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 0, 0, 1, 0},
            {0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 0, 1, 0, 0},
            {0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
            {0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
            {0, 0, 0, 0, 0, 1, 0, 0, 0, 0}};
      double[][] distance = new double[][]{
            {0, 92.1443, 0, 0, 0, 0, 0, 0, 0, 0},
            {95.8048, 0, 123.294, 122.041, 0, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 62.0565, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 120.095, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 0, 0, 120.648, 0},
            {0, 77.2222, 0, 0, 0, 0, 0, 0, 0, 0},
            {0, 0, 0, 0, 0, 0, 0, 139.407, 0, 0},
            {0, 0, 0, 0, 0, 103.207, 0, 0, 0, 0},
            {0, 149.863, 0, 0, 0, 0, 0, 0, 0, 93.1061},
            {0, 0, 0, 0, 0, 92.7064, 0, 0, 0, 0}};
      return new Topology(N, connectivity, distance);
   }
   
   private static Topology topology_1(){
      int N=10;
      int[][] connectivity = new int[][]{
         {1, 1, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 1, 1, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 1, 0, 0, 0, 1, 0},
         {0, 0, 0, 0, 1, 0, 0, 0, 0, 1},
         {1, 1, 0, 0, 0, 1, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 1, 0, 0, 0},
         {1, 1, 0, 0, 0, 0, 0, 1, 0, 0},
         {0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
         {0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 0, 1, 0, 0}};
      double[][] distance = new double[][]{
         {0, 79.9984, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 61.1778, 112.024, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 108.847, 0, 0, 0, 120.189, 0},
         {0, 0, 0, 0, 85.0481, 0, 0, 0, 0, 98.8281},
         {73.2846, 125.286, 0, 0, 0, 149.188, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 101.893, 0, 0, 0},
         {96.1043, 120.871, 0, 0, 0, 0, 0, 89.9272, 0, 0},
         {0, 0, 0, 0, 0, 121.109, 0, 0, 0, 0},
         {0, 116.873, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 0, 88.3404, 0, 0}};
      return new Topology(N, connectivity, distance);
   }
   
   private static Topology topology_2(){
      int N=10;
      int[][] connectivity = new int[][]{
         {1, 1, 1, 1, 1, 0, 0, 0, 0, 0},
         {0, 0, 1, 0, 1, 1, 0, 0, 0, 0},
         {1, 0, 0, 0, 0, 0, 1, 0, 0, 0},
         {0, 0, 1, 0, 0, 0, 0, 0, 1, 0},
         {1, 0, 1, 0, 0, 1, 0, 0, 0, 1},
         {0, 1, 0, 0, 0, 0, 0, 0, 1, 0},
         {0, 0, 0, 0, 0, 1, 0, 1, 0, 0},
         {0, 0, 1, 0, 1, 1, 0, 0, 0, 0},
         {1, 0, 0, 0, 0, 0, 1, 0, 0, 0},
         {1, 0, 0, 0, 0, 1, 0, 0, 1, 0}};
      double[][] distance = new double[][]{
         {0, 79.1219, 105.492, 93.0807, 134.027, 0, 0, 0, 0, 0},
         {0, 0, 109.345, 0, 102.715, 104.559, 0, 0, 0, 0},
         {114.429, 0, 0, 0, 0, 0, 125.074, 0, 0, 0},
         {0, 0, 127.553, 0, 0, 0, 0, 0, 85.6671, 0},
         {104.608, 0, 76.1119, 0, 0, 69.9958, 0, 0, 0, 87.787},
         {0, 133.388, 0, 0, 0, 0, 0, 0, 76.5066, 0},
         {0, 0, 0, 0, 0, 87.8381, 0, 113.244, 0, 0},
         {0, 0, 81.2783, 0, 97.2317, 103.037, 0, 0, 0, 0},
         {101.163, 0, 0, 0, 0, 0, 106.259, 0, 0, 0},
         {102.737, 0, 0, 0, 0, 104.708, 0, 0, 148.463, 0}};
      return new Topology(N, connectivity, distance);
   }
   
   private static Topology topology_3(){
      int N=10;
      int[][] connectivity = new int[][]{
         {1, 1, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 1, 0, 0, 0, 0, 0},
         {0, 0, 0, 1, 1, 1, 1, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 1, 1, 0, 0},
         {1, 0, 0, 1, 0, 0, 1, 0, 0, 0},
         {0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
         {1, 0, 1, 0, 1, 0, 0, 0, 0, 1},
         {0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
         {0, 0, 1, 0, 1, 0, 0, 0, 1, 0}};
      double[][] distance = new double[][]{
         {0, 77.3676, 0, 0, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 118.233, 0, 0, 0, 0, 0},
         {0, 0, 0, 111.32, 95.609, 83.1417, 102.272, 0, 0, 0},
         {0, 0, 0, 0, 0, 0, 53.6793, 129.817, 0, 0},
         {115.354, 0, 0, 93.507, 0, 0, 124.638, 0, 0, 0},
         {0, 0, 0, 65.3991, 0, 0, 98.4519, 122.849, 0, 0},
         {95.7125, 0, 109.628, 0, 124.998, 0, 0, 0, 0, 78.9011},
         {0, 0, 0, 0, 0, 121.644, 0, 0, 0, 0},
         {0, 0, 0, 0, 0, 93.4218, 0, 0, 0, 125.015},
         {0, 0, 99.9288, 0, 93.0729, 0, 0, 0, 114.702, 0}};
      return new Topology(N, connectivity, distance);
   }
   
   private static Topology topology_4(){
      int N=10;
      int[][] connectivity = new int[][]{
         {1, 1, 1, 1, 1, 0, 0, 0, 0, 0},
         {0, 0, 0, 1, 1, 0, 0, 1, 0, 0},
         {1, 0, 0, 1, 1, 1, 0, 0, 0, 0},
         {0, 0, 0, 0, 1, 0, 0, 0, 0, 1},
         {0, 1, 0, 1, 0, 0, 1, 0, 0, 0},
         {0, 1, 0, 0, 0, 0, 1, 0, 0, 0},
         {0, 0, 1, 0, 1, 0, 0, 0, 0, 0},
         {0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
         {1, 1, 0, 0, 1, 1, 0, 0, 0, 0},
         {0, 0, 0, 0, 1, 0, 0, 1, 1, 0}};
      double[][] distance = new double[][]{
         {0, 95.4206, 90.2338, 117.011, 80.9737, 0, 0, 0, 0, 0},
         {0, 0, 0, 69.8323, 102.155, 0, 0, 110.528, 0, 0},
         {153.085, 0, 0, 89.3526, 83.1885, 93.4606, 0, 0, 0, 0},
         {0, 0, 0, 0, 83.445, 0, 0, 0, 0, 82.9012},
         {0, 73.3017, 0, 100.772, 0, 0, 98.0976, 0, 0, 0},
         {0, 51.995, 0, 0, 0, 0, 92.6622, 0, 0, 0},
         {0, 0, 86.5808, 0, 120.807, 0, 0, 0, 0, 0},
         {0, 0, 0, 97.8119, 0, 0, 0, 0, 0, 0},
         {53.1308, 122.731, 0, 0, 110.945, 69.9604, 0, 0, 0, 0},
         {0, 0, 0, 0, 83.2344, 0, 0, 115.057, 88.0052, 0}};
      return new Topology(N, connectivity, distance);
   }
   
   private static Topology topology_5(){
      int N=10;
      int[][] connectivity = new int[][]{
         {1, 1, 1, 0, 0, 0, 0, 0, 0, 0},
         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0},
         {0, 1, 0, 0, 0, 0, 1, 1, 0, 0},
         {1, 1, 0, 0, 0, 0, 0, 0, 1, 0},
         {0, 0, 0, 0, 0, 1, 1, 0, 0, 0},
         {0, 1, 1, 1, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 1, 0, 0, 1, 0, 0},
         {1, 1, 1, 0, 0, 1, 1, 0, 0, 0},
         {0, 0, 1, 1, 0, 1, 0, 0, 0, 1},
         {0, 0, 1, 0, 1, 0, 0, 1, 0, 0}};
      double[][] distance = new double[][]{
         {0, 90.9987, 59.6363, 0, 0, 0, 0, 0, 0, 0},
         {63.8723, 0, 0, 111.254, 0, 0, 0, 0, 0, 0},
         {0, 89.5961, 0, 0, 0, 0, 83.6933, 122.703, 0, 0},
         {105.922, 140.684, 0, 0, 0, 0, 0, 0, 101.311, 0},
         {0, 0, 0, 0, 0, 83.8463, 125.379, 0, 0, 0},
         {0, 117.658, 99.8206, 100.245, 0, 0, 0, 0, 0, 0},
         {0, 0, 0, 0, 84.034, 0, 0, 104.451, 0, 0},
         {96.9527, 132.35, 78.2505, 0, 0, 110.19, 132.201, 0, 0, 0},
         {0, 0, 71.8607, 112.457, 0, 129.257, 0, 0, 0, 98.6789},
         {0, 0, 53.3002, 0, 83.9608, 0, 0, 70.0929, 0, 0}};
      return new Topology(N, connectivity, distance);
   }
}
